package com.qmth.teachcloud.exchange.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.tools.signature.SignatureEntity;
import com.qmth.boot.tools.signature.SignatureType;
import com.qmth.teachcloud.exchange.common.bean.params.OpenParams;
import com.qmth.teachcloud.exchange.common.contant.SystemConstant;
import com.qmth.teachcloud.exchange.common.entity.BasicSchool;
import com.qmth.teachcloud.exchange.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.exchange.common.service.BasicSchoolService;
import com.qmth.teachcloud.exchange.common.service.CommonService;
import com.qmth.teachcloud.exchange.common.util.Base64Util;
import com.qmth.teachcloud.exchange.common.util.JacksonUtil;
import com.qmth.teachcloud.exchange.common.util.ServletUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Objects;
import java.util.Optional;
import java.util.StringJoiner;

/**
 * @Description: 公用serviceImpl 实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/25
 */
@Service
public class CommonServiceImpl implements CommonService {
    private final static Logger log = LoggerFactory.getLogger(CommonServiceImpl.class);

    @Resource
    BasicSchoolService basicSchoolService;

    @Value("${cas.config.teachcloudPrintLoginUrl}")
    String teachcloudLoginUrl;

    /**
     * 跳转方法
     *
     * @param account
     * @param schoolCode
     * @param returnUrl
     * @param params
     * @throws IOException
     */
    @Override
    public void redirectLogic(String account, String schoolCode, String returnUrl, String params) throws IOException {
        log.info("公用跳转接口进来了,account:{},schoolCode:{},returnUrl:{},params:{}", account, schoolCode, returnUrl, params);
        if (Objects.isNull(teachcloudLoginUrl) || Objects.equals(teachcloudLoginUrl, "")) {
            throw ExceptionResultEnum.PARAMS_ERROR.exception("知学登录跳转地址不存在");
        }
        if (Objects.isNull(account) || Objects.equals(account, "")) {
            throw ExceptionResultEnum.PARAMS_ERROR.exception("工号不存在");
        }
        if (Objects.isNull(schoolCode) || Objects.equals(schoolCode, "")) {
            throw ExceptionResultEnum.PARAMS_ERROR.exception("学校code不存在");
        }
//        accountUser = URLDecoder.decode(accountUser, SystemConstant.CHARSET_NAME);

        QueryWrapper<BasicSchool> basicSchoolQueryWrapper = new QueryWrapper<>();
        basicSchoolQueryWrapper.lambda().eq(BasicSchool::getCode, schoolCode);
        BasicSchool basicSchool = basicSchoolService.getOne(basicSchoolQueryWrapper);
        Optional.ofNullable(basicSchool).orElseThrow(() -> ExceptionResultEnum.PARAMS_ERROR.exception("学校信息不存在"));

        if (teachcloudLoginUrl.contains(SystemConstant.PATH_MATCH)) {
            teachcloudLoginUrl = teachcloudLoginUrl.replace(SystemConstant.PATH_MATCH, basicSchool.getCode());
        }

        long timestamp = System.currentTimeMillis();
        String path = teachcloudLoginUrl.substring(teachcloudLoginUrl.indexOf(SystemConstant.PATH_SUBSTR), teachcloudLoginUrl.length());
        String signature = SignatureEntity.build(SignatureType.SECRET, SystemConstant.GET, path, timestamp, basicSchool.getAccessKey(), basicSchool.getAccessSecret());

        StringJoiner stringJoiner = new StringJoiner("")
                .add(teachcloudLoginUrl)
                .add(SystemConstant.GET_UNKNOWN).add(SystemConstant.HEADER_TIME).add(SystemConstant.GET_EQUAL).add(String.valueOf(timestamp))
                .add(SystemConstant.GET_SYMBOL).add(SystemConstant.SIGNATURE).add(SystemConstant.GET_EQUAL).add(Base64Util.encode(signature.getBytes(StandardCharsets.UTF_8)))
                .add(SystemConstant.GET_SYMBOL).add(SystemConstant.USER).add(SystemConstant.GET_EQUAL).add(URLEncoder.encode(account, SystemConstant.CHARSET_NAME));

        if (Objects.nonNull(returnUrl) && !Objects.equals(returnUrl, "")) {
            stringJoiner = stringJoiner.add(SystemConstant.GET_SYMBOL).add(SystemConstant.RETURN_URL).add(SystemConstant.GET_EQUAL).add(URLEncoder.encode(returnUrl, SystemConstant.CHARSET_NAME));
        }
        if (Objects.nonNull(params) && !Objects.equals(params, "")) {
            String decodeJson = URLDecoder.decode(params, SystemConstant.CHARSET_NAME);
            OpenParams openParams = JacksonUtil.readJson(decodeJson, OpenParams.class);
            stringJoiner = stringJoiner.add(SystemConstant.GET_SYMBOL).add(SystemConstant.PARAMS).add(SystemConstant.GET_EQUAL).add(URLEncoder.encode(JacksonUtil.parseJson(openParams), SystemConstant.CHARSET_NAME));
        }
//        String redirectUrl = URLEncoder.encode(stringJoiner.toString(), SystemConstant.CHARSET);
//        log.info("path:{},timestamp:{},account:{},signature:{},redirectUrl:{},redirectUrl_urlencode:{}", path, timestamp, account, signature, stringJoiner.toString(), redirectUrl);
        log.info("path:{},timestamp:{},account:{},signature:{}, redirectUrl:{}", path, timestamp, account, signature, stringJoiner.toString());

        HttpServletResponse response = ServletUtil.getResponse();
        response.setHeader("Access-Control-Allow-Origin", SystemConstant.PATH_MATCH);
        response.sendRedirect(stringJoiner.toString());
    }
}
